class MMBounds
  require 'MMLocation'
  require 'MMLatLon'

  def initialize( south_west = nil, north_east = nil )
    self.reset( south_west, north_east )
  end

  def reset( south_west = nil, north_east = nil )
    @south_west = nil
    @north_east = nil

    if south_west.class == Array
      south_west.each do |e|
        self.extend e
      end
    elsif south_west || north_east
      sw = self._getPoint( south_west )
      ne = self._getPoint( north_east )
      # If a user passes one point, or nil, point in to the constructor
      # then only one of sw or ne will be set, rectify that here.
      if( ! sw )
        sw = ne
      end
      if( ! ne )
        ne = sw
      end
      @south_west = ( sw ? sw.copy() : sw )
      @north_east = ( ne ? ne.copy() : ne )
    end
  end

  # Allows the bounds object to be created from a javascript object literal.
  # @param {Object} json_bounds A JS object containing a set of bounds.
  def fromJSON( json_bounds )
    if json_bounds && json_bounds['SouthWest'] 
      @south_west = MMLatLon.new(json_bounds['SouthWest']['Lat'].to_f, json_bounds['SouthWest']['Lon'].to_f); 
      @north_east = MMLatLon.new(json_bounds['NorthEast']['Lat'].to_f, json_bounds['NorthEast']['Lon'].to_f)
    elsif json_bounds json_bounds['south_west']
      @south_west = MMLatLon.new(json_bounds['south_west']['lat'].to_f, json_bounds['south_west']['lon'].to_f)
      @north_east = MMLatLon.new(json_bounds['north_east']['lat'].to_f, json_bounds['north_east']['lon'].to_f)
    end
  end

  # Function to return a simple string representation of the object.
  # @return Formatted version of the object as "[(swlat,swlon), (nelat,nelon)]".
  # @type String
  def to_s
    return '[ '+(@south_west ? @south_west.to_s : 'nil')+', '+(@north_east ? @north_east.to_s : 'nil' )+' ]'
  end

  # Function to return a copy of the object in a new object.
  # @return A new object containing the same data as self object.
  # @type MMLocation
  def copy
    cp = MMBounds.new
    if @south_west
      cp.south_west = @south_west.copy
    end
    if @north_east
      cp.north_east = @north_east.copy
    end
    return cp
  end


  # Extends the bounds to contain the given point.
  # Extends the bounds by the smallest amount possible.
  #
  # @param {Object} point Point to extend the bounds by.
  def extend( point )
    if( ! point )
      return
    end
    if point.class == MMBounds
      self.extend( point.south_west )
      self.extend( point.north_east )
      return
    end
    point = self._getPoint( point )
    if ! @south_west
      @south_west = point.copy()
      @north_east = point.copy()
    else
      if point.class == MMPoint
        if( point.x < @south_west.x )
          @south_west.x = point.x
        end
        if( point.y < @south_west.y )
          @south_west.y = point.y
        end
        if( point.x > @north_east.x )
          @north_east.x = point.x
        end
        if( point.y > @north_east.y )
          @north_east.y = point.y
        end
      else
        if( point.lat < @south_west.lat )
          @south_west.lat = point.lat
        end
        if( point.lat > @north_east.lat )
          @north_east.lat = point.lat
        end
        possibles = []
        contained = self.contains( point )
        if ! contained
          # Try using the lon of the new point as a replacement for
          # each of the SW and NE lons and get the new distance
          possibles.push( {
            :distance => self._getDistance( @south_west.lon, point.lon ), 
            :sw_lon => @south_west.lon,
            :ne_lon => point.lon } )
          possibles.push( {
            :distance => self._getDistance( point.lon, @north_east.lon ), 
            :sw_lon => point.lon,
            :ne_lon => @north_east.lon } )

          # The one with the shortest distance will be the one we want
          shortest_distance = nil
          possibles.each do |p|
            if shortest_distance.nil? || p[:distance] < shortest_distance
              @south_west.lon = p[:sw_lon]
              @north_east.lon = p[:ne_lon]
              shortest_distance = p[:distance]
            end
          end
        end
      end
    end
  end

  # Extracts the coordinates from the various possible objects
  # that can be passed in.
  #
  # @param {Object} point A MMMarkerOverlay, MMLocation, MMLatLon
  # @returns The coordinates for the point 
  # @type MMLatLon
  # @private
  def _getPoint( point )
    if( point && point.class == MMLocation )
      point = point.coords
    end
    if point
      point.forceNumbers()
    end
    return point
  end

  # Returns true if the rectangle bounds contain the given point.
  #
  # @param {Object} point Point to check for.
  # @returns True if the point is within the bounds, false otherwise.
  # @type boolean
  def contains( point )
    contained = false

    if @south_west.class == MMLatLon
      puts "a"
      if @south_west.lon > @north_east.lon
        if( ( point.lon >= @south_west.lon || point.lon <= @north_east.lon ) && point.lat >= @south_west.lat && point.lat <= @north_east.lat )
          contained = true
        end
      else
      puts "b"
        if( ( point.lon >= @south_west.lon && point.lon <= @north_east.lon ) && point.lat >= @south_west.lat && point.lat <= @north_east.lat )
      puts "c"
          contained = true
        end
      end
    end
    if( @south_west.class == MMPoint )
      if( point.x >= @south_west.x && point.x <= @north_east.x && point.y >= @south_west.y && point.y <= @north_east.y )
        contained = true
      end
    end
    return contained
  end

  # Gets the 'distance' between 2 lons, isn't a correct geographical distance
  # just something to be used for working over the IDL.
  # @private
  def _getDistance( sw_lon, ne_lon )
    if( sw_lon > ne_lon )
      return ( ne_lon + 360 ) - sw_lon
    end
    return ne_lon - sw_lon
  end

  # Gets the point at the southwest of the rectangle.
  #
  # @returns The point at the southwest corner of the rectangle.
  # @type MMLatLon
  def getSouthWest
    if @south_west
      return @south_west.copy
    end
    return nil
  end

  # Gets the point at the northeast of the rectangle.
  #
  # @returns The point at the northeast corner of the rectangle.
  # @type MMLatLon
  def getNorthEast
    if @north_east
      return @north_east.copy
    end
    return nil
  end

  # Gets the point at the southeast of the rectangle.
  #
  # @returns The point at the southeast corner of the rectangle.
  # @type MMLatLon
  def getSouthEast
    if @north_east.class == MMLatLon
      return MMLatLon.new( @south_west.lat, @north_east.lon )
    end
    if @north_east.class == MMPoint
      return MMPoint.new( @north_east.x, @south_west.y )
    end
  end

  # Gets the point at the northwest of the rectangle.
  #
  # @returns The point at the northwest corner of the rectangle.
  # @type MMLatLon
  def getNorthWest
    if @north_east.class == MMLatLon
      return MMLatLon.new( @north_east.lat, @south_west.lon )
    end
    if @north_east.class == MMPoint
      return MMPoint.new( @south_west.x, @north_east.y )
    end
  end

  # Returns true if self bounds rectangle contains the given bounds rectangle.
  #
  # @param {MMBounds} other The other rectangle to check.
  # @returns Returns true if self bounds rectangle contains the given bounds rectangle.
  # @type boolean
  def containsBounds( other )
    return ( self.contains( other.getSouthWest() ) && self.contains( other.getNorthEast() ) )
  end

  # Returns true if self bounds rectangle intersects the given bounds rectangle.
  #
  # @param {MMBounds} other The other rectangle to check.
  # @returns Returns true if self bounds rectangle intersects the given bounds rectangle.
  # @type boolean
  def intersects( other )
    if( @south_west.class == MMLatLon )
      # If one point is over the IDL we need to do two checks,
      #   one where the rightmost point is moved to the right,
      #   one where the leftmost point is moved to the left.
      # If two points are over, they can both be shifted to the
      #   right and that's all.
      left1 = @south_west.lon
      right1 = @north_east.lon
      left2 = other.getSouthWest.lon
      right2 = other.getNorthEast.lon
      intersection = true
      idl = false
      if( @south_west.lon > @north_east.lon )
        right1 += 360
        idl = true
      end
      if( other.getSouthWest.lon > other.getNorthEast.lon )
        right2 += 360
        idl = ! idl
      end

      if( left1 > right2 )
        intersection = false
      end
      if( intersection && right1 < left2 )
        intersection = false
      end

      if( idl && ! intersection )
        intersection = true
        if( @south_west.lon > @north_east.lon )
          right1 -= 360
          left1 -= 360
        end
        if( other.getSouthWest.lon > other.getNorthEast.lon )
          right2 -= 360
          left1 -= 360
        end
        if( left1 > right2 )
          intersection = false
        end
        if( intersection && right1 < left2 )
          intersection = false
        end
      end
      return intersection
    else
      if( @south_west.x > other.getNorthEast.x )
        return false
      end
      if( @south_west.y > other.getNorthEast.y )
        return false
      end
      if( @north_east.x < other.getSouthWest.x )
        return false
      end
      if( @north_east.y < other.getSouthWest.y )
        return false
      end
    end

    return true
  end

  # Retrieves the center point of the bounded rectangle.
  #
  # @returns The center point of the bounded rectangle.
  # @type MMLatLon
  def getCenter
    sw = self.getSouthWest()
    ne = self.getNorthEast()
    sw.forceNumbers()
    ne.forceNumbers()
    center = nil
    if( sw.class == MMLatLon )
      if( sw.lon > ne.lon )
        ne.lon += 360
      end
      center = MMLatLon.new(
        sw.lat + ( (ne.lat - sw.lat ) / 2 ),
        sw.lon + ( (ne.lon - sw.lon ) / 2 ) )
      if( center.lon > 180 )
        center.lon -= 360
      end
    else
      center = MMPoint.new(
        sw.x + ( ( ne.x - sw.x ) / 2 ),
        sw.y + ( ( ne.y - sw.y ) / 2 ) )
    end
    return center
  end

  # Retrieves the appropritate REST query string for a bounding box search.
  # 
  # @returns The string to be appended to the URL
  # @type String
  # @private
  def toAPIQueryString(postfix_number)
    var postfix = ''
    if (postfix_number)
      postfix = '_' + postfix_number
    end
    return "&bb" + postfix + "=" + self.getSouthWest().lon + ',' + self.getSouthWest().lat + ';' + self.getNorthEast().lon + ',' + self.getNorthEast().lat
  end
end
