require 'test/unit'
require 'MMBounds'

class MMBoundsTest < Test::Unit::TestCase

  # Replace this with your real tests.
  def test_truth
    assert true
  end

  # Just switches the args around
  def assert_equals( a, b, c )
    return assert_equal( b, c, a )
  end

  def assert_true( a, b )
    return assert( b, a )
  end

  def testSimpleConstructor
    bounds1 = MMBounds.new( MMLatLon.new( -10.0, -10.0 ), MMLatLon.new( 10.0, 10.0 ) )
    bounds2 = MMBounds.new( MMLatLon.new( -10.0, -10.0 ) )
    bounds3 = MMBounds.new( nil, MMLatLon.new( 10.0, 10.0 ) )
    bounds4 = MMBounds.new( MMLocation.new( :coords => MMLatLon.new( -10.0, -10.0 ) ), MMLocation.new( :coords => MMLatLon.new( 10.0, 10.0 ) ) )
    bounds5 = MMBounds.new( MMLatLon.new( -10.0, 10.0 ), MMLatLon.new( 10.0, -10.0 ) )
    assert_equals( "Basic assignment of SW broken.", '(-10.0,-10.0)', bounds1.getSouthWest.to_s )
    assert_equals( "Basic assignment of NE broken.", '(10.0,10.0)', bounds1.getNorthEast.to_s )
    assert_equals( "Assignment with single argument broken.", bounds2.getSouthWest.to_s, bounds2.getNorthEast.to_s )
    assert_equals( "Assignment with undefined first argument broken.", bounds3.getSouthWest.to_s, bounds3.getNorthEast.to_s )
    assert_equals( "Basic MMLocation assignment of SW broken.", '(-10.0,-10.0)', bounds4.getSouthWest.to_s )
    assert_equals( "Basic MMLocation assignment of NE broken.", '(10.0,10.0)', bounds4.getNorthEast.to_s )
    assert_equals( "Basic assignment of IDL SW broken.", '(-10.0,10.0)', bounds5.getSouthWest.to_s )
    assert_equals( "Basic assignment of IDL NE broken.", '(10.0,-10.0)', bounds5.getNorthEast.to_s )
  end

  def testExtendingNormal
    bounds = MMBounds.new( MMLatLon.new( -10.0, -10.0 ), MMLatLon.new( 10.0, 10.0 ) )
    bounds.extend( MMLatLon.new( 15, 15 ) )
    assert_equals( "Extending NE didn't work.", '[ (-10.0,-10.0), (15.0,15.0) ]', bounds.to_s )
    bounds.extend( MMLatLon.new( 20, -15 ) )
    assert_equals( "Extending W and N didn't work.", '[ (-10.0,-15.0), (20.0,15.0) ]', bounds.to_s )
    bounds.extend( MMLatLon.new( -15, 7 ) )
    assert_equals( "Third extension didn't move S correctly.", '[ (-15.0,-15.0), (20.0,15.0) ]', bounds.to_s )

    bounds = MMBounds.new( MMLatLon.new( 53.36663, -2.90809 ), MMLatLon.new( 53.38286, -2.88785 ) )
    bounds.extend( MMLatLon.new( 53.382867, -2.90807 ) )
    assert_equals( "Known bad extension failed again.", '[ (53.36663,-2.90809), (53.382867,-2.88785) ]', bounds.to_s )
  end

  def testExtendingIDL
    bounds = MMBounds.new( MMLatLon.new( -10.0, 50.0 ), MMLatLon.new( 10.0, -50.0 ) )
    bounds.extend( MMLatLon.new( 0, 100.0 ) )
    assert_equals( "Adding point within IDL bounds extended bounds when it shouldn't.", '[ (-10.0,50.0), (10.0,-50.0) ]', bounds.to_s )
    bounds.extend( MMLatLon.new( 0, -100.0 ) )
    assert_equals( "Adding negative lon point within IDL bounds extended bounds when it shouldn't.", '[ (-10.0,50.0), (10.0,-50.0) ]', bounds.to_s )
    bounds.extend( MMLatLon.new( 0, 40.0 ) )
    assert_equals( "Expanding bounds to west.", '[ (-10.0,40.0), (10.0,-50.0) ]', bounds.to_s )
    bounds.extend( MMLatLon.new( 0, -40.0 ) )
    assert_equals( "Expanding bounds to east.", '[ (-10.0,40.0), (10.0,-40.0) ]', bounds.to_s )

    bounds2 = MMBounds.new( MMLatLon.new( -10.0, 170 ), MMLatLon.new( 10.0, 179 ) )
    assert_equals( "Normal bounds object creation", '[ (-10.0,170.0), (10.0,179.0) ]', bounds2.to_s )
    bounds2.extend( MMLatLon.new( 4, -175 ) )
    assert_equals( "Expanding a normal bounds object across the dateline", '[ (-10.0,170.0), (10.0,-175.0) ]', bounds2.to_s )
  end

  def testPointsConstructor
    bounds = MMBounds.new( [ MMLatLon.new( 10.0, 20 ), MMLatLon.new( -5, 13 ), MMLatLon.new( 23, -5 ), MMLatLon.new( -7, 3 ) ] )
    assert_equals( "Basic set of points", '[ (-7.0,-5.0), (23.0,20.0) ]', bounds.to_s )
    idlbounds = MMBounds.new( [ MMLatLon.new( 10.0, 150.0 ), MMLatLon.new( 20, -170 ), MMLatLon.new( -54, 170 ) ] )
    assert_equals( "IDL Points", '[ (-54.0,150.0), (20.0,-170.0) ]', idlbounds.to_s )
  end

  def testIntersects
    # A number of intersection tests

    # A set of bounds 10.0x10.0 around the origin
    bounds1 = MMBounds.new( MMLatLon.new( -5, -5 ), MMLatLon.new( 5, 5 ) )

    # Each of these is a set of bounds with one corner in the test bounds
    boundssw = MMBounds.new( MMLatLon.new( 0, 0 ), MMLatLon.new( 10.0, 10.0 ) )
    boundsnw = MMBounds.new( MMLatLon.new( -10.0, 0 ), MMLatLon.new( 0, 10.0 ) )
    boundsne = MMBounds.new( MMLatLon.new( -10.0, -10.0 ), MMLatLon.new( 0, 0 ) )
    boundsse = MMBounds.new( MMLatLon.new( 0, -10.0 ), MMLatLon.new( 10.0, 0 ) )
    assert_true( "SW bounds intersects test bounds", bounds1.intersects( boundssw ) )
    assert_true( "NW bounds intersects test bounds", bounds1.intersects( boundsnw ) )
    assert_true( "NE bounds intersects test bounds", bounds1.intersects( boundsne ) )
    assert_true( "SE bounds intersects test bounds", bounds1.intersects( boundsse ) )

    # Smaller than the test bounds, i.e. within
    boundssmall = MMBounds.new( MMLatLon.new( -2, -2 ), MMLatLon.new( 2, 2 ) )
    assert_true( "Small bounds intersects test bounds", bounds1.intersects( boundssmall ) )

    # Larger than the test bounds, i.e. around
    boundsbig = MMBounds.new( MMLatLon.new( -10.0, -10.0 ), MMLatLon.new( 10.0, 10.0 ) )
    assert_true( "Big bounds intersects test bounds", bounds1.intersects( boundsbig ) )

    # Each of these intersects the test bounds on a side, not a corner
    boundsw = MMBounds.new( MMLatLon.new( -10.0, 0 ), MMLatLon.new( 10.0, 10.0 ) )
    boundsn = MMBounds.new( MMLatLon.new( -10.0, -10.0 ), MMLatLon.new( 0, 10.0 ) )
    boundse = MMBounds.new( MMLatLon.new( -10.0, -10.0 ), MMLatLon.new( 10.0, 0 ) )
    boundss = MMBounds.new( MMLatLon.new( 0, -10.0 ), MMLatLon.new( 10.0, 10.0 ) )
    assert_true( "W bounds intersects test bounds", bounds1.intersects( boundsw ) )
    assert_true( "N bounds intersects test bounds", bounds1.intersects( boundsn ) )
    assert_true( "E bounds intersects test bounds", bounds1.intersects( boundse ) )
    assert_true( "S bounds intersects test bounds", bounds1.intersects( boundss ) )

    # Smaller than the test bounds, adjacent
    boundssmalladj = MMBounds.new( MMLatLon.new( -2, -12 ), MMLatLon.new( 2, -8 ) )
    assert_true( "Small adjacent bounds intersects test bounds", ! bounds1.intersects( boundssmalladj ) )

    # Bigger than the test bounds, adjacent
    boundsbigadj = MMBounds.new( MMLatLon.new( -10.0, -20 ), MMLatLon.new( 10.0, -10.0 ) )
    assert_true( "Big adjacent bounds intersects test bounds", ! bounds1.intersects( boundsbigadj ) )

    # Tall and narrow vs wide and short
    boundstall = MMBounds.new( MMLatLon.new( -1, 0 ), MMLatLon.new( 10.0, 1 ) )
    boundswide = MMBounds.new( MMLatLon.new( 0, -1 ), MMLatLon.new( 1, 10.0 ) )
    assert_true( "Tall and narrow vs wide and short", boundstall.intersects( boundswide ) )

    # Intersection across the date line
    boundsidl1 = MMBounds.new( MMLatLon.new( -10.0, 170 ), MMLatLon.new( 10.0, -170 ) )
    boundsidl2 = MMBounds.new( MMLatLon.new( -5, -175 ), MMLatLon.new( 5, -160 ) )
    boundsidl3 = MMBounds.new( MMLatLon.new( -5, 160 ), MMLatLon.new( 5, 175 ) )
    boundsidl4 = MMBounds.new( MMLatLon.new( 0, 175 ), MMLatLon.new( 20, -165 ) )
    assert_true( "Intersection across the date line, one across, one not (on the west)", boundsidl1.intersects( boundsidl2 ) )
    assert_true( "Intersection across the date line, one across, one not (on the east)", boundsidl1.intersects( boundsidl3 ) )
    assert_true( "Intersection across the date line, both across", boundsidl1.intersects( boundsidl4 ) )
    
  end

  def testGetCenter
    bounds1 = MMBounds.new( MMLatLon.new( -10.0, 150.0 ), MMLatLon.new( 10.0, 170 ) )
    bounds2 = MMBounds.new( MMLatLon.new( "-10.0", "150.0" ), MMLatLon.new( "10.0", "170" ) )
    assert_equals( "getCenter returns correctly using numbers", "(0.0,160.0)", bounds1.getCenter.to_s )
    assert_equals( "getCenter returns incorrectly using strings", "(0.0,160.0)", bounds2.getCenter.to_s )
  end
end
