class MMGeneral
  include Math
  def self.latLonToMercator( lat, lon, eccentricity = nil )

    k0 = 1        # scale factor at the natural origin.    Unity in our case
    lon0 = 0        # longitude of the natural origin.    Is the prime meridian in our case

    rlat = lat * (Math::PI / 180)        # convert from degrees to radians
    rlon = lon * (Math::PI / 180)

    a = 6378137                             # radius of WGS84 ellipsoid 
    f = 298.257223563     # reciprocal flattening of ellipsoid
    e = 0.0818191908426215        # eccentricity of the ellipsoid
    if eccentricity
      e = eccentricity
    end


    b1 = (Math::PI/4) + (rlat/2)
    b1 = Math::tan(b1)

    if e
      b2 = (1 - (e * Math::sin(rlat))) / (1 + (e * Math::sin(rlat)))
      b2 = b2 ** (e / 2)
    else
      b2 = 1
    end

    b = b1 * b2

    # only find log if number is greater than zero - flog an error otherwise
    if (b>0)
        ln = Math::log(b)
    else
        ln = 0
    end

    y = a * k0 * ln
    x = a * k0 * (rlon - lon0)

    return { :x => x, :y => y }
  end

  def self.coordToTile( x, y, zoom )
    world_width = 40075016.0
    half_world_width = 20037508.0
    digits = zoom - 1
    tile_width = world_width / ( 2 ** digits ).to_f
    x = (x + half_world_width)/tile_width
    y = (y + half_world_width)/tile_width
    return { :x => x, :y => y }
  end

  def self.tileToMorton( x, y, zoom )
    morton = '';
    d = zoom - 1;
    x = x.to_i
    y = y.to_i
    while d > 1
      d = d - 1
      morton += ( (y>>d).to_s+(x>>d).to_s ).to_i(2).to_s;
      x %= ( 2 ** d );
      y %= ( 2 ** d );
    end
    if zoom != 1 
      morton += ( y.to_s+x.to_s ).to_i(2).to_s;
    end
    return morton
  end

  def self.latLonToMorton( lat, lon, e, zoom )
    merc = self.latLonToMercator( lat, lon, e )
    tile = self.coordToTile( merc[:x], merc[:y], zoom )
    return self.tileToMorton( tile[:x], tile[:y], zoom )
  end
end
