class MMLatLon
  attr_accessor :lat, :lon

  def initialize( lat = nil, lon = nil )
    self.lat = lat
    self.lon = lon
  end

  def copy
    cp = MMLatLon.new
    cp.lat = self.lat
    cp.lon = self.lon
    return cp
  end

  def toAPIQueryString( postfix = "" )
    url = ""
    postfix = postfix.to_s
    if postfix != ""
      postfix = "_"+postfix
    end
    if self.lat
      url += "&lat"+postfix+"="+URI.escape(self.lat.to_s,/[^0-9a-zA-Z\-]/)
    end
    if self.lon
      url += "&lon"+postfix+"="+URI.escape(self.lon.to_s,/[^0-9a-zA-Z\-]/)
    end
    return url
  end

  def to_json(*a)
    {                                                                                                                                                                                               
      "lat" => self.lat,
      "lon" => self.lon
    }.to_json(*a)
  end

  def to_s
    return "("+self.lat.to_s+","+self.lon.to_s+")"
  end

  def latToCanonical
    return self.toCanonical( self.lat, 'N', 'S' )
  end

  def lonToCanonical
    return self.toCanonical( self.lon, 'E', 'W' )
  end

  def toCanonical( val, a, b )
    ns = a
    if val < 0
        val = -val
        ns = b
    end
    val = ( val * 3600 + 0.5).to_i.to_f
    deg = ( val/3600 ).to_i
    min = ((val-3600*(val / 3600).to_i.to_f)/60).to_i
    sec = (val - 60*(val/60).to_i.to_f).to_i
    if min < 10 
      min = '0'+min.to_s
    else
      min = min.to_s
    end
    if sec < 10 
        sec = '0'+sec.to_s
    else
      sec = sec.to_s
    end
    return deg.to_s+':'+min+':'+sec+ns
  end

  def fromJSON( json )
    if json
      self.lat = json['lat'].to_f
      self.lon = json['lon'].to_f
    end
  end

  def forceNumbers
    self.lat = self.lat.to_f
    self.lon = self.lon.to_f
    return self
  end

  def truncate
    if self.lat
      self.lat = ( self.lat * 100000 ).to_i.to_f / 100000
    end
    if self.lon
      self.lon = ( self.lon * 100000 ).to_i.to_f / 100000
    end
    return self
  end
end
