# A simple object to describe a point that has X and Y values.
# Generally used for giving co-ordinates in pixels, such as to moveMap.
#
# @param {Object} x The x value for the point.
# @param {Object} y The y value for the point.
class MMPoint
  attr_accessor :x, :y

  def initialize( x, y )
    # The x value for the point.
    # @type Object
    self.x = x;
    # The y value for the point.
    # @type Object
    self.y = y;
  end

  # Function to return a simple string representation of the object.
  # @return Formatted version of the object as "(x,y)".
  # @type String
  def to_s
    return '('+self.x.to_s+','+self.y.to_s+')'
  end

  # Function to make a copy of the object in a new object.
  # @return A new object containing the same data as self object.
  # @type MMPoint
  def copy
    return MMPoint.new( self.x, self.y )
  end

  # Function to ensure the attributes stored are floating point numbers.
  # 
  def forceNumbers
    self.x = self.x.to_f
    self.y = self.y.to_f
    return self
  end

  def toAPIQueryString( postfix = "" )
    url = ""
    postfix = postfix.to_s
    if postfix != ""
      postfix = "_"+postfix
    end
    if self.x
      url += "&x"+postfix+"="+URI.escape(self.x.to_s,/[^0-9a-zA-Z\-]/)
    end
    if self.y
      url += "&y"+postfix+"="+URI.escape(self.y.to_s,/[^0-9a-zA-Z\-]/)
    end
    return url
  end

   
end
