# The Multimap API Searching module - JavaScript library.
# <br />JavaScript library for accessing the functionality of the Multimap API Searching module.
# <br />Requires access to the Multimap API Searching module - please contact your Multimap account manager for further information.
# <br />The input parameter can be one of the following:
# <ul><li>A JSON object to initialize MMSearch with. See field summary for further information.</li>
# <li>A MMBounds object to use as the bounding_box field</li>
# <li>A MMAddress object to use as the address field</li>
# <li>A MMSearchFilter object which will be the only element in the filters array</li>
# <li>A MMLatLon object to be used as the point field</li>
# <li>A MMSearch object to be used as the pre_search field</li></ul>
# @param {Object} init_search Described above.
class MMSearch
  require 'MMLatLon'
  require 'MMBounds'
  require 'MMAddress'
  #require 'MMRoute'
  require 'MMSearchFilter'
  attr_accessor :data_source, :endpoint, :count, :format, :start_index, :bounding_box, :point, :address, :pre_search, :max_distance, :min_distance, :distance_units, :order_by_fields, :order_by_order, :return_fields, :filters, :logic, :compound_order, :compound_threshold, :result_set_size, :morton, :route_modes, :return_infobox

  def initialize( init_search = {})
    # The data source to be queried when the search executes. Can be a comma-separated list searching multiple data sources in one request. Each data source will have its own record set returned.
    # <BR/>Default: Data source of Multimap API client name
    # @type String
    if init_search[:data_source]
      self.data_source = init_search[:data_source]
    end

    self.endpoint = nil
     
    # Specifies the number of records to return.
    # <BR/>Default: 20
    # @type integer
    self.count = nil

    # Specifies whether to return all record details or only the count of the records returned.
    # <BR/>Permitted values: 'normal', 'compact'
    # <BR/>Default: 'normal'
    # @type String
    self.format = nil

    # Specifies the index of the first record to display.
    # <BR/>Default: 1
    # @type integer
    self.start_index = nil
      
    # Specifies a bounding box that limits the area in which a search can take place.
    # <BR/>Default: none
    # @type MMBounds
    self.bounding_box = nil
      
    # Specifies a lat/lon point to search from.
    # <BR/>Default: none
    # @type MMLatLon
    self.point = nil
      
    # Specifies an address to search from.
    # <BR/>Default: none
    # @type MMAddress
    self.address = nil
      
    # A MMSearch object whose results can be used as the source for another search.
    # <BR/>Default: none
    # @type MMSearch
    self.pre_search = nil
      
    # Limits the search results to those within a certain distance of the point, address or pre_search.
    # <BR/>Default: none
    # @type float
    self.max_distance = nil
      
    # Limits the search results to those outside a certain distance from the point or address.
    # <BR/>Default: none
    # @type float
    self.min_distance = nil
      
    # Specifies the units to use in conjunction with max_distance and min_distance.
    # <BR/>Permitted values: 'miles', 'km'
    # <BR/>Default: 'miles'
    # @type String
    self.distance_units = nil
      
    # Array of strings containing fields to be used to order the search results.
    # Can also be a comma-separated list of fields.
    # <BR/>Default: 'distance'
    # @type Array
    self.order_by_fields = nil
      
    # Array of order direction values to match up with fields provided in order_by_fields.  Can also be a comma-separated list.
    # <BR/>Permitted values: 'asc', 'desc'
    # <BR/>Default: 'asc'
    # @type Array
    self.order_by_order = nil
      
    # Array of fields to return in the search results. Can also be a comma-separated list.
    # <BR/>Default: all fields
    # @type Array
    self.return_fields = nil
      
    # Array of MMSearchFilter for filtering results using non-spatial values.
    # <BR/>Default: none
    # @type Array
    self.filters = nil
      
    # Comma separated list of logic operator to use when more than one value for 'filters' is provided.
    # <BR/>Permitted values: 'AND', 'OR'
    # <BR/>Default: 'AND'
    # @type String
    self.logic = nil

    # Comma separated list specifying the order in which individual operators should be applied when using the "com" (compound) search operator within an MMSearchFilter.
    # <BR/>Permitted values: eq, contains, startswith, endswith, like, sl, co
    # <BR/>Default: 'eq,contains,sl'
    # @type String
    self.compound_order = nil
      
    # Specifies the number of records the compound search operator should try to retrieve. If set to 0, the compound query will halt as soon as one of the individual operators matches at least 1 row. If set to 10, the compound query will continue to run until at least 10 matches are found, or there are no more operators to use.
    # <BR/>Default: none
    # @type integer
    self.compound_threshold = nil


    # Used when point queries also have order by settings to first get a result set of the specified size sorted by distance from the point before doing the order bys.
    # <BR/>Default: none
    # @type integer
    self.result_set_size = nil


    self.morton = nil

    # Comma separated list specifying routing modes, calculates and adds routed distances and durations to all search results.
    # <BR/>Permitted values: driving, walking
    # <BR/>Default: 'eq,contains,sl'
    # @type String
    self.route_modes = nil

    
# @private
    @search_attributes = [
      :data_source,:count,:format,:start_index,:bounding_box,
      :point,:address,:max_distance,:distance_units,:min_distance,
      :morton,:order_by_fields,:order_by_order,:return_fields,
      :filters,:pre_search,:result_set_size,:compound_order,
      :compound_threshold,:logic,:return_infobox,:route_modes,
      :route,:endpoint ]

    if init_search.is_a?( MMBounds )
      self.bounding_box = init_search
    elsif init_search.is_a?( MMSearchFilter )
      self.filters = [ init_search ]
    elsif init_search.is_a?(MMAddress)
      self.address = init_search
    elsif init_search.is_a?(MMLatLon)
      self.point = init_search
    elsif init_search.is_a?(MMSearch)
      self.pre_search = init_search
    elsif init_search 
      @search_attributes.each do |sa|
        puts sa
        if init_search[sa]
          puts sa.to_s+"="+init_search[sa].inspect
          self.instance_variable_set( '@'+sa.to_s, init_search[sa] )
        end
      end
    end

  end

# Makes a new MMSearch object containing a copy of the data within this one.
#
# @type MMSearch
# @returns A new MMSearch object containing a copy of the data within this one.
  def copy
    cp = MMSearch.new
    @search_attribute.each do |sa|
      if self.instance_variable_get(sa)
        cp.instance_variable_set(sa,self.instance_variable_get(sa))
      end
    end
    return cp
  end

# Function to reset this object before performing another search.
# @private
  def reset
    @search_attribute.each do |sa|
      self.instance_variable_set(sa,nil)
    end
  end

# Function to return a simple string representation of the object.
# @return A stringified dump of the attributes of the object.
# @type String
  def to_s
    ret = ''
    
    @search_attribute.each do |sa|
      if self.instance_variable_get(sa)
        ret += sa + ": "+ self.instance_variable_get(sa)+ " "
      end
    end
    return ret
  end

# Function to return a partial query string based on the stored attributes
# for use with Multimap's Search Web Service.
# @return Partial query string based on the stored attributes.
# @type String
# @private
  def toAPIQueryString( postfix_number = nil )
    postfix = ''
    if postfix_number
      postfix = '_' + postfix_number
    end
    query_string = ''
    
    if self.data_source
      if self.data_source.is_a?(Array)
        query_string += "&dataSource" + postfix + "=" + self.data_source.join(',')
      else
        query_string += "&dataSource" + postfix + "=" + self.data_source
      end
    end
        
    if self.count
      query_string += "&count" + postfix + "=" + self.count
    end
    
    if self.format
      query_string += "&format" + postfix + "=" + self.format
    end
        
    if self.start_index
      query_string += "&startIndex" + postfix + "=" + self.start_index
    end
                
    if self.bounding_box
      query_string += self.bounding_box.toAPIQueryString(postfix_number)
    end
        
    if self.point
      query_string += self.point.toAPIQueryString(postfix_number)
    end

    if self.address
      query_string += self.address.toAPIQueryString(postfix_number)
    end
        
    if self.max_distance
      query_string += "&maxDistance" + postfix + "=" + self.max_distance
    end
    
    if self.distance_units
      query_string += "&distanceUnits" + postfix + "=" + self.distance_units
    end
        
    if self.logic
      query_string += "&logic" + postfix + "=" + self.logic
    end
        
    if self.min_distance
      query_string += "&minDistance" + postfix + "=" + self.min_distance
    end
    
    if self.order_by_fields
      if self.order_by_fields.is_a?(Array)
        query_string += "&orderByFields" + postfix + "=" + self.order_by_fields.join(',')
      else
        query_string += "&orderByFields" + postfix + "=" + self.order_by_fields
      end
    end
    
    if self.order_by_order
      if self.order_by_order.is_a?(Array)
        query_string += "&orderByOrder" + postfix + "=" + self.order_by_order.join(',')
      else
        query_string += "&orderByOrder" + postfix + "=" + self.order_by_order
      end
    end
    
    if self.return_fields
      if self.return_fields.is_a?(Array)
        query_string += "&returnFields" + postfix + "=" + self.return_fields.join(',')
      else
        query_string += "&returnFields" + postfix + "=" + self.return_fields
      end
    end

    if self.return_infobox
        query_string += "&returnInfobox=" + self.return_infobox
    end

    preflight_filters = 0
    if self.pre_search
      if self.pre_search.data_source
        query_string += "&preSearchDataSource=" + self.pre_search.data_source
      end
      if self.pre_search.search_first_alt
        query_string += "&searchFirstAlt=" + self.pre_search.search_first_alt
      end
      if self.pre_search.filters
        # handle array of MMFieldFilter objects
        self.pre_search.filters.each do |f|
          query_string += f.toAPIQueryString(i + 1)
          preflight_filters = i
        end
      end
      preflight_filters += 1
      query_string += '&preSearchFilters=' + preflight_filters
    end
    
    if self.compound_threshold
      query_string += "&compoundThreshold" + postfix + "=" + self.compound_threshold
    end

    if self.result_set_size
      query_string += "&resultSetSize" + postfix + "=" + self.result_set_size
    end

    if self.morton
      query_string += "&morton" + postfix + "=" + self.morton
    end

    if self.compound_order
      query_string += "&compoundOrder" + postfix + "=" + self.compound_order
    end

    if self.route_modes
      query_string += "&routeModes=" + self.route_modes
    end

    #if self.route && self.route.is_a?(MMRoute)
    #  query_string += self.route.toAPIQueryString()
    #end

    if self.filters
      # handle array of MMFieldFilter objects
      self.filters.each do |f|
        # MMSearch objects with multiple field filters won't work for preflight routing
        if postfix_number
          postfix_number -= 1
        else
          postfix_number = 0
        end
        query_string += f.toAPIQueryString(i + 1 + preflight_filters + postfix_number)
      end
    end

    return query_string
  end
end
