# The Multimap API Searching module - JavaScript library.
# <br />JavaScript library for accessing the functionality of the Multimap API Searching module
# <br />Requires access to the Multimap API Searching module - please contact your Multimap account manager for further information
# @param {Function} callback Function to call when the search completes.
class MMSearchRequester
  require 'json'
  require 'MMSearch'

# Attempts to perform a search; calls the callback function on completion. 
# The search object exposes two attributes that can be retrieved by
# the callback function. The first is record_set. The second is error_code which will be
# undefined if no error occurred, otherwise a valid error code.
#
# @param {MMSearch} MMSearch object containing the search criteria to use. 
  def self.search( search )
    if search.nil?
      return
    end
    
    path_str = '/API/search/1.2/'+MM_CLIENT_NAME+'?output=json'+search.toAPIQueryString()
    puts path_str
    res = Net::HTTP.start( MM_HOST_NAME, 80 ) { |http|
      http.get(path_str)
    }
    json = JSON.parse( res.body )
    
    if json['error_code']
        # service error
        return json['error_code']
        #this['error_explanation'] = json['error_explanation'];        
    end
    if json['errors']
        return 'Pre-flight errors'
        # preflight errors - should really be handled better
        
    end
    if json['record_sets']
      # for each record set
      results = []
      json['record_sets'].each do |ds,r|
        record_set = self.parse_record_set(r, ds)
        results.push(record_set);                
      end
    end

    return results
  end

  def self.parse_record_set(record_set, datasource)

    record_set['dataSource'] = datasource

    if record_set['total_record_count']
        record_set['totalRecordCount'] = record_set['total_record_count']
    end
    
    if record_set['records']
      # for each record
      record_set['records'].each do |record|

        if record['point']
          record['point'] = MMLatLon.new(record['point']['lat'].to_f, record['point']['lon'].to_f)
        end
      end
    end

    return record_set
  end

end
